# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!! 
# This file is built by ./mktables from e.g. UnicodeData.txt.
# Any changes made here will be lost!

#
# This file supports:
# 	\p{Thai} (and fuzzy permutations)
# 
# Meaning: Script 'Thai'
#
return <<'END';
0E01	0E3A	Thai
0E40	0E5B	Thai
END
